@@ -1,7 +1,7 @@
 #!/usr/bin/python
 #
 
-# Copyright (C) 2006, 2007, 2008, 2010, 2012 Google Inc.
+# Copyright (C) 2006, 2007, 2008, 2010, 2012, 2013 Google Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,6 +22,7 @@
 """Script for unittesting the objects module"""
 
 
+import copy
 import unittest
 
 from ganeti import constants
@@ -223,6 +224,37 @@ class TestClusterObject(unittest.TestCase):
     self.assertRaises(errors.ConfigurationError, cluster.UpgradeConfig)
 
 
+class TestClusterObjectTcpUdpPortPool(unittest.TestCase):
+  def testNewCluster(self):
+    self.assertTrue(objects.Cluster().tcpudp_port_pool is None)
+
+  def testSerializingEmpty(self):
+    self.assertEqual(objects.Cluster().ToDict(), {
+      "tcpudp_port_pool": [],
+      })
+
+  def testSerializing(self):
+    cluster = objects.Cluster.FromDict({})
+    self.assertEqual(cluster.tcpudp_port_pool, set())
+
+    cluster.tcpudp_port_pool.add(3546)
+    cluster.tcpudp_port_pool.add(62511)
+
+    data = cluster.ToDict()
+    self.assertEqual(data.keys(), ["tcpudp_port_pool"])
+    self.assertEqual(sorted(data["tcpudp_port_pool"]), sorted([3546, 62511]))
+
+  def testDeserializingEmpty(self):
+    cluster = objects.Cluster.FromDict({})
+    self.assertEqual(cluster.tcpudp_port_pool, set())
+
+  def testDeserialize(self):
+    cluster = objects.Cluster.FromDict({
+      "tcpudp_port_pool": [26214, 10039, 267],
+      })
+    self.assertEqual(cluster.tcpudp_port_pool, set([26214, 10039, 267]))
+
+
 class TestOS(unittest.TestCase):
   ALL_DATA = [
     "debootstrap",
@@ -383,40 +415,164 @@ class TestInstancePolicy(unittest.TestCase):
 
   def _AssertIPolicyIsFull(self, policy):
     self.assertEqual(frozenset(policy.keys()), constants.IPOLICY_ALL_KEYS)
-    for key in constants.IPOLICY_ISPECS:
-      spec = policy[key]
-      self.assertEqual(frozenset(spec.keys()), constants.ISPECS_PARAMETERS)
+    self.assertTrue(len(policy[constants.ISPECS_MINMAX]) > 0)
+    for minmax in policy[constants.ISPECS_MINMAX]:
+      self.assertEqual(frozenset(minmax.keys()), constants.ISPECS_MINMAX_KEYS)
+      for key in constants.ISPECS_MINMAX_KEYS:
+        self.assertEqual(frozenset(minmax[key].keys()),
+                         constants.ISPECS_PARAMETERS)
+    self.assertEqual(frozenset(policy[constants.ISPECS_STD].keys()),
+                     constants.ISPECS_PARAMETERS)
 
   def testDefaultIPolicy(self):
     objects.InstancePolicy.CheckParameterSyntax(constants.IPOLICY_DEFAULTS,
                                                 True)
     self._AssertIPolicyIsFull(constants.IPOLICY_DEFAULTS)
 
+  def _AssertPolicyIsBad(self, ipolicy, do_check_std=None):
+    if do_check_std is None:
+      check_std_vals = [False, True]
+    else:
+      check_std_vals = [do_check_std]
+    for check_std in check_std_vals:
+      self.assertRaises(errors.ConfigurationError,
+                        objects.InstancePolicy.CheckISpecSyntax,
+                        ipolicy, check_std)
+
   def testCheckISpecSyntax(self):
+    default_stdspec = constants.IPOLICY_DEFAULTS[constants.ISPECS_STD]
+    incomplete_ipolicies = [
+      {
+         constants.ISPECS_MINMAX: [],
+         constants.ISPECS_STD: default_stdspec,
+         },
+      {
+         constants.ISPECS_MINMAX: [{}],
+         constants.ISPECS_STD: default_stdspec,
+         },
+      {
+        constants.ISPECS_MINMAX: [{
+          constants.ISPECS_MIN: NotImplemented,
+          }],
+        constants.ISPECS_STD: default_stdspec,
+        },
+      {
+        constants.ISPECS_MINMAX: [{
+          constants.ISPECS_MAX: NotImplemented,
+          }],
+        constants.ISPECS_STD: default_stdspec,
+        },
+      {
+        constants.ISPECS_MINMAX: [{
+          constants.ISPECS_MIN: NotImplemented,
+          constants.ISPECS_MAX: NotImplemented,
+          }],
+        },
+      ]
+    for ipol in incomplete_ipolicies:
+      self.assertRaises(errors.ConfigurationError,
+                        objects.InstancePolicy.CheckISpecSyntax,
+                        ipol, True)
+      oldminmax = ipol[constants.ISPECS_MINMAX]
+      if oldminmax:
+        # Prepending valid specs shouldn't change the error
+        ipol[constants.ISPECS_MINMAX] = ([constants.ISPECS_MINMAX_DEFAULTS] +
+                                         oldminmax)
+        self.assertRaises(errors.ConfigurationError,
+                          objects.InstancePolicy.CheckISpecSyntax,
+                          ipol, True)
+
+    good_ipolicy = {
+      constants.ISPECS_MINMAX: [
+        {
+          constants.ISPECS_MIN: {
+            constants.ISPEC_MEM_SIZE: 64,
+            constants.ISPEC_CPU_COUNT: 1,
+            constants.ISPEC_DISK_COUNT: 2,
+            constants.ISPEC_DISK_SIZE: 64,
+            constants.ISPEC_NIC_COUNT: 1,
+            constants.ISPEC_SPINDLE_USE: 1,
+            },
+          constants.ISPECS_MAX: {
+            constants.ISPEC_MEM_SIZE: 16384,
+            constants.ISPEC_CPU_COUNT: 5,
+            constants.ISPEC_DISK_COUNT: 12,
+            constants.ISPEC_DISK_SIZE: 1024,
+            constants.ISPEC_NIC_COUNT: 9,
+            constants.ISPEC_SPINDLE_USE: 18,
+            },
+          },
+        {
+          constants.ISPECS_MIN: {
+            constants.ISPEC_MEM_SIZE: 32768,
+            constants.ISPEC_CPU_COUNT: 8,
+            constants.ISPEC_DISK_COUNT: 1,
+            constants.ISPEC_DISK_SIZE: 1024,
+            constants.ISPEC_NIC_COUNT: 1,
+            constants.ISPEC_SPINDLE_USE: 1,
+            },
+          constants.ISPECS_MAX: {
+            constants.ISPEC_MEM_SIZE: 65536,
+            constants.ISPEC_CPU_COUNT: 10,
+            constants.ISPEC_DISK_COUNT: 5,
+            constants.ISPEC_DISK_SIZE: 1024 * 1024,
+            constants.ISPEC_NIC_COUNT: 3,
+            constants.ISPEC_SPINDLE_USE: 12,
+            },
+          },
+        ],
+      }
+    good_ipolicy[constants.ISPECS_STD] = copy.deepcopy(
+      good_ipolicy[constants.ISPECS_MINMAX][0][constants.ISPECS_MAX])
+    # Check that it's really good before making it bad
+    objects.InstancePolicy.CheckISpecSyntax(good_ipolicy, True)
+
+    bad_ipolicy = copy.deepcopy(good_ipolicy)
+    for minmax in bad_ipolicy[constants.ISPECS_MINMAX]:
+      for (key, spec) in minmax.items():
+        for param in spec:
+          oldv = spec[param]
+          del spec[param]
+          self._AssertPolicyIsBad(bad_ipolicy)
+          if key == constants.ISPECS_MIN:
+            spec[param] = minmax[constants.ISPECS_MAX][param] + 1
+          self._AssertPolicyIsBad(bad_ipolicy)
+          spec[param] = oldv
+    assert bad_ipolicy == good_ipolicy
+
+    stdspec = bad_ipolicy[constants.ISPECS_STD]
+    for param in stdspec:
+      oldv = stdspec[param]
+      del stdspec[param]
+      self._AssertPolicyIsBad(bad_ipolicy, True)
+      # Note that std spec is the same as a max spec
+      stdspec[param] = oldv + 1
+      self._AssertPolicyIsBad(bad_ipolicy, True)
+      stdspec[param] = oldv
+    assert bad_ipolicy == good_ipolicy
+
+    for minmax in good_ipolicy[constants.ISPECS_MINMAX]:
+      for spec in minmax.values():
+        good_ipolicy[constants.ISPECS_STD] = spec
+        objects.InstancePolicy.CheckISpecSyntax(good_ipolicy, True)
+
+  def testCheckISpecParamSyntax(self):
     par = "my_parameter"
     for check_std in [True, False]:
-      if check_std:
-        allkeys = constants.IPOLICY_ISPECS
-      else:
-        allkeys = constants.IPOLICY_ISPECS - frozenset([constants.ISPECS_STD])
-      # Only one policy limit
-      for key in allkeys:
-        policy = dict((k, {}) for k in allkeys)
-        policy[key][par] = 11
-        objects.InstancePolicy.CheckISpecSyntax(policy, par, check_std)
       # Min and max only
       good_values = [(11, 11), (11, 40), (0, 0)]
       for (mn, mx) in good_values:
-        policy = dict((k, {}) for k in allkeys)
-        policy[constants.ISPECS_MIN][par] = mn
-        policy[constants.ISPECS_MAX][par] = mx
-        objects.InstancePolicy.CheckISpecSyntax(policy, par, check_std)
-      policy = dict((k, {}) for k in allkeys)
-      policy[constants.ISPECS_MIN][par] = 11
-      policy[constants.ISPECS_MAX][par] = 5
+        minmax = dict((k, {}) for k in constants.ISPECS_MINMAX_KEYS)
+        minmax[constants.ISPECS_MIN][par] = mn
+        minmax[constants.ISPECS_MAX][par] = mx
+        objects.InstancePolicy._CheckISpecParamSyntax(minmax, {}, par,
+                                                     check_std)
+      minmax = dict((k, {}) for k in constants.ISPECS_MINMAX_KEYS)
+      minmax[constants.ISPECS_MIN][par] = 11
+      minmax[constants.ISPECS_MAX][par] = 5
       self.assertRaises(errors.ConfigurationError,
-                        objects.InstancePolicy.CheckISpecSyntax,
-                        policy, par, check_std)
+                        objects.InstancePolicy._CheckISpecParamSyntax,
+                        minmax, {}, par, check_std)
     # Min, std, max
     good_values = [
       (11, 11, 11),
@@ -424,29 +580,34 @@ class TestInstancePolicy(unittest.TestCase):
       (11, 40, 40),
       ]
     for (mn, st, mx) in good_values:
-      policy = {
+      minmax = {
         constants.ISPECS_MIN: {par: mn},
-        constants.ISPECS_STD: {par: st},
         constants.ISPECS_MAX: {par: mx},
         }
-      objects.InstancePolicy.CheckISpecSyntax(policy, par, True)
+      stdspec = {par: st}
+      objects.InstancePolicy._CheckISpecParamSyntax(minmax, stdspec, par, True)
     bad_values = [
-      (11, 11,  5),
-      (40, 11, 11),
-      (11, 80, 40),
-      (11,  5, 40),
-      (11,  5,  5),
-      (40, 40, 11),
+      (11, 11,  5, True),
+      (40, 11, 11, True),
+      (11, 80, 40, False),
+      (11,  5, 40, False,),
+      (11,  5,  5, True),
+      (40, 40, 11, True),
       ]
-    for (mn, st, mx) in bad_values:
-      policy = {
+    for (mn, st, mx, excp) in bad_values:
+      minmax = {
         constants.ISPECS_MIN: {par: mn},
-        constants.ISPECS_STD: {par: st},
         constants.ISPECS_MAX: {par: mx},
         }
-      self.assertRaises(errors.ConfigurationError,
-                        objects.InstancePolicy.CheckISpecSyntax,
-                        policy, par, True)
+      stdspec = {par: st}
+      if excp:
+        self.assertRaises(errors.ConfigurationError,
+                          objects.InstancePolicy._CheckISpecParamSyntax,
+                          minmax, stdspec, par, True)
+      else:
+        ret = objects.InstancePolicy._CheckISpecParamSyntax(minmax, stdspec,
+                                                            par, True)
+        self.assertFalse(ret)
 
   def testCheckDiskTemplates(self):
     invalid = "this_is_not_a_good_template"
@@ -468,18 +629,14 @@ class TestInstancePolicy(unittest.TestCase):
   def testCheckParameterSyntax(self):
     invalid = "this_key_shouldnt_be_here"
     for check_std in [True, False]:
-      self.assertRaises(KeyError,
-                        objects.InstancePolicy.CheckParameterSyntax,
-                        {}, check_std)
-      policy = objects.MakeEmptyIPolicy()
-      policy[invalid] = None
+      objects.InstancePolicy.CheckParameterSyntax({}, check_std)
+      policy = {invalid: None}
       self.assertRaises(errors.ConfigurationError,
                         objects.InstancePolicy.CheckParameterSyntax,
                         policy, check_std)
       for par in constants.IPOLICY_PARAMETERS:
-        policy = objects.MakeEmptyIPolicy()
         for val in ("blah", None, {}, [42]):
-          policy[par] = val
+          policy = {par: val}
           self.assertRaises(errors.ConfigurationError,
                             objects.InstancePolicy.CheckParameterSyntax,
                             policy, check_std)
@@ -499,7 +656,7 @@ class TestInstancePolicy(unittest.TestCase):
   def _AssertIPolicyMerged(self, default_pol, diff_pol, merged_pol):
     for (key, value) in merged_pol.items():
       if key in diff_pol:
-        if key in constants.IPOLICY_ISPECS:
+        if key == constants.ISPECS_STD:
           self._AssertISpecsMerged(default_pol[key], diff_pol[key], value)
         else:
           self.assertEqual(value, diff_pol[key])
@@ -511,6 +668,9 @@ class TestInstancePolicy(unittest.TestCase):
       {constants.IPOLICY_VCPU_RATIO: 3.14},
       {constants.IPOLICY_SPINDLE_RATIO: 2.72},
       {constants.IPOLICY_DTS: [constants.DT_FILE]},
+      {constants.ISPECS_STD: {constants.ISPEC_DISK_COUNT: 3}},
+      {constants.ISPECS_MINMAX: [constants.ISPECS_MINMAX_DEFAULTS,
+                                 constants.ISPECS_MINMAX_DEFAULTS]}
       ]
     for diff_pol in partial_policies:
       policy = objects.FillIPolicy(constants.IPOLICY_DEFAULTS, diff_pol)
@@ -518,22 +678,13 @@ class TestInstancePolicy(unittest.TestCase):
       self._AssertIPolicyIsFull(policy)
       self._AssertIPolicyMerged(constants.IPOLICY_DEFAULTS, diff_pol, policy)
 
-  def testFillIPolicySpecs(self):
-    partial_policies = [
-      {constants.ISPECS_MIN: {constants.ISPEC_MEM_SIZE: 32},
-       constants.ISPECS_MAX: {constants.ISPEC_CPU_COUNT: 1024}},
-      {constants.ISPECS_STD: {constants.ISPEC_DISK_SIZE: 2048},
-       constants.ISPECS_MAX: {
-          constants.ISPEC_DISK_COUNT: constants.MAX_DISKS - 1,
-          constants.ISPEC_NIC_COUNT: constants.MAX_NICS - 1,
-          }},
-      {constants.ISPECS_STD: {constants.ISPEC_SPINDLE_USE: 3}},
-      ]
-    for diff_pol in partial_policies:
-      policy = objects.FillIPolicy(constants.IPOLICY_DEFAULTS, diff_pol)
-      objects.InstancePolicy.CheckParameterSyntax(policy, True)
-      self._AssertIPolicyIsFull(policy)
-      self._AssertIPolicyMerged(constants.IPOLICY_DEFAULTS, diff_pol, policy)
+  def testFillIPolicyKeepsUnknown(self):
+    INVALID_KEY = "invalid_ipolicy_key"
+    diff_pol = {
+      INVALID_KEY: None,
+      }
+    policy = objects.FillIPolicy(constants.IPOLICY_DEFAULTS, diff_pol)
+    self.assertTrue(INVALID_KEY in policy)
 
 
 if __name__ == "__main__":
